# Step 1

from PIL import Image

# Load the uploaded image
image_path = "/mnt/data/Screenshot 2024-12-06 at 4.31.38 AM.png"
image = Image.open(image_path)

# Display the image for verification
image.show()

# Step 2

import cv2
import numpy as np
import matplotlib.pyplot as plt

# Load the image using OpenCV for analysis
image_cv = cv2.imread(image_path)
# Convert the image to RGB format (OpenCV loads in BGR by default)
image_rgb = cv2.cvtColor(image_cv, cv2.COLOR_BGR2RGB)

# Display the image for visual inspection
plt.figure(figsize=(10, 10))
plt.imshow(image_rgb)
plt.axis('off')
plt.title("Uploaded Image for Analysis")
plt.show()

# Step 3

# Convert the image to HSV for better color segmentation
image_hsv = cv2.cvtColor(image_cv, cv2.COLOR_BGR2HSV)

# Define the orange color range for mitochondria detection
lower_orange = np.array([5, 100, 100])  # Lower bound of orange in HSV
upper_orange = np.array([25, 255, 255])  # Upper bound of orange in HSV

# Create a mask for orange regions
orange_mask = cv2.inRange(image_hsv, lower_orange, upper_orange)

# Perform morphological operations to clean up noise
kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (5, 5))
cleaned_mask = cv2.morphologyEx(orange_mask, cv2.MORPH_CLOSE, kernel)

# Find contours for the mitochondria
contours, _ = cv2.findContours(cleaned_mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

# Draw contours on a blank image for visualization
contour_image = np.zeros_like(image_cv)
cv2.drawContours(contour_image, contours, -1, (0, 255, 0), 2)

# Display the contours for verification
plt.figure(figsize=(10, 10))
plt.imshow(cv2.cvtColor(contour_image, cv2.COLOR_BGR2RGB))
plt.axis('off')
plt.title(f"Detected Mitochondria: {len(contours)} Total")
plt.show()

# Count mitochondria per cell (approximate based on location grouping)
len(contours)  # Total mitochondria detected

# Step 4

# Approximate grouping of mitochondria into cells based on centroid locations
# Find contours' centroids
centroids = [cv2.moments(c) for c in contours]
centroid_coords = [(int(c["m10"] / c["m00"]), int(c["m01"] / c["m00"])) for c in centroids if c["m00"] != 0]

# Manual clustering based on the approximate positions of the three cells
# Divide the image roughly into three regions based on x-coordinates
image_width = image_cv.shape[1]
cell_boundaries = [
    (0, image_width // 3),  # Left cell
    (image_width // 3, 2 * image_width // 3),  # Middle cell
    (2 * image_width // 3, image_width),  # Right cell
]

# Count mitochondria within each region
cell_counts = [0, 0, 0]  # Initialize counts for the three cells
for x, y in centroid_coords:
    for i, (start, end) in enumerate(cell_boundaries):
        if start <= x < end:
            cell_counts[i] += 1
            break

cell_counts  # Mitochondria counts for each cell
